;; lissajous.scm -*-scheme-*-
;;
;; Script for drawing Lissajous figures and harmonographs

;; Script at http://members.optusnet.com.au/~charles57/GIMP
;; Version 1.0
;;
;; Copyright (C) 2008 by Charles Cave <charlesweb@optusnet.com.au>
;;
;; This program is free software; you can redistribute it and/or
;; modify it under the terms of the GNU General Public License
;; as published by the Free Software Foundation; either version 2
;; of the License, or (at your option) any later version.
;;
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program; if not, write to the Free Software
;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (script-fu-lissajous       img drw
                                   margin
                                   amult bmult
                                   phaseoffset stepsize
                                   decaypct
                                   cycles
                                   tool brush color)
    (let* (
             (t 0)
             ;; one more step needed to draw up to starting point
             (maxt (+ (* cycles 360) stepsize)) 

             (thisx 0)
             (thisy 0)
             (lastx 0)
             (lasty 0)
             (curxrad 0)       ;; xrad adjusted with decaypct
             (curyrad 0)       ;; xrad adjusted with decaypct
             (imgwidth  (car (gimp-image-width  img)))
             (imgheight (car (gimp-image-height img)))
             (xcenter (/ imgwidth 2))
             (ycenter (/ imgheight 2))
             (xrad (/ (- imgwidth  (* 2 margin)) 2))
             (yrad (/ (- imgheight (* 2 margin)) 2))
             (*linepoints* (cons-array 4 'double))
          )
 
   (gimp-context-push)
   (gimp-image-undo-group-start img)
 
   (gimp-context-set-foreground color)
   (gimp-context-set-brush (car brush))
   (gimp-context-set-opacity (car (cdr brush)))
   (gimp-context-set-paint-mode (car (cdr (cdr (cdr brush)))))

   ;; radians = (degress * PI ) / 180
   (define (deg2rad x) (/ (* 3.14159265 x) 180))  
    
    (while (< t maxt)
       ;; The decay percentage is used to reduce the value of xrad and yrad
       ;; the valie of t from 0 to 360 is one revolution so if decaypct is < 100
       ;; more cycles will be needed. For example, if decaypct is 20%, then 5
       ;; revolutions will be needed.
           
       ;; xrad = xrad - ( t / 360) * xrad * ( decay / 100 )
       (set! curxrad (* xrad (- 1 (* (/ t 360) (/ decaypct 100)))))
       (set! curyrad (* yrad (- 1 (* (/ t 360) (/ decaypct 100)))))
       (if (< curxrad 0) (set! curxrad 0))
       (if (< curyrad 0) (set! curyrad 0))

       ;; calculate   x = A * sin(at + offset)       
        (set! thisx (+ xcenter (* curxrad (sin  (deg2rad (+ (* amult t) phaseoffset))))))       
       ;; calculate  y = B * sin(bt)
       (set! thisy (+ ycenter (* curyrad (sin (deg2rad (* bmult t))))))

       (if (> t 0)
           (begin
            (aset *linepoints* 0 lastx) 
            (aset *linepoints* 1 lasty)
            (aset *linepoints* 2 thisx) 
            (aset *linepoints* 3 thisy) 
            (if (= tool 0)
                (gimp-pencil drw 4 *linepoints*)           ; Use pencil
                (if (= tool 1)
                    (gimp-paintbrush-default drw 4 *linepoints*); use paintbrush
                    (gimp-airbrush-default drw 4 *linepoints*)  ; use airbrush
                    ))))
        
       (set! t (+ t stepsize))
       (set! lasty thisy)
       (set! lastx thisx)
    )

    (gimp-image-undo-group-end img)
    (gimp-displays-flush)   (gimp-context-pop)
 ))

(script-fu-register "script-fu-lissajous"
                    _"Lissajous Drawing"
                    _"Draws issajous figures and Harmonographs"
                    "Charles Cave <charlesweb@optusnet.com.au>"
                    "Charles Cave"
                    "September 2008"
                    "RGB*, INDEXED*, GRAY*"
                    SF-IMAGE       "Image"         0
                    SF-DRAWABLE    "Drawable"      0
                    SF-ADJUSTMENT _"Margin"       '(5 0 100 1 10 0 1)  
                    SF-ADJUSTMENT _"A multiplier" '(3 1 200 1 10 0 1)
                    SF-ADJUSTMENT _"B multiplier" '(1 1 200 1 10 0 1)
                    SF-ADJUSTMENT _"Phase Offset (degrees) " '(0 -180 180 1 10 0 0)
                    SF-ADJUSTMENT _"Step size (degrees)" '(1 1 60 1 10 0 0)
                    SF-ADJUSTMENT _"Decay Percentage"  '(0 0 100 1 10 0 0)
                    SF-ADJUSTMENT _"Cycles" '(1 1 100 1 10 0 0)
                    SF-OPTION     _"Tool"          '(_"Pencil"
                                  _"Brush"
                                  _"Airbrush")

                    SF-BRUSH      _"Brush"         '("Circle (01)" 100 1 0)
                    SF-COLOR      _"Color"          "black"
                    )

(script-fu-menu-register "script-fu-lissajous"
                         "<Image>/Filters")
